"use strict";

exports.__esModule = true;
exports.atom = atom;
exports.useAtom = useAtom;
var _react = require("react");
/**
 * An atom represents a global state. The atom() function returns a Atom object.
 *
 * The Atom manages the state changes to know when to notify components subscribing to that store to re-render,
 * so you should use the useStore hook to acess the store state into a react component.
 *
 * The setState method of a atom can be called outside of react components
 *
 * This API is deeply inspired by atom from Recoil
 *
 * @see https://recoiljs.org/docs/api-reference/core/atom/
 */
function atom(initialState) {
  var state = initialState;
  var listeners = new Set();
  return {
    setState: function setState(valueOrUpdater) {
      if (valueOrUpdater instanceof Function) {
        state = valueOrUpdater(state);
      } else {
        state = valueOrUpdater;
      }
      listeners.forEach(function (listener) {
        listener();
      });
    },
    getState: function getState() {
      return state;
    },
    subscribe: function subscribe(listener) {
      listeners.add(listener);
      return function () {
        listeners["delete"](listener);
      };
    }
  };
}

/**
 * This hook will subscribe the component to re-render for any changes in the requested state.
 *
 * Returns a tuple where the first element is the value of state and
 * the second element is a setter function that will update the value
 * of the given state when called.
 *
 * @param atom
 * @returns [state, setState]
 * @example
 *
 * const counterAtom = atom(0)
 *
 * // use it only inside a react component
 * const [counter, setCounter] = useAtom(counterAtom)
 *
 * This API is deeply inspired by useRecoilState from Recoil
 *
 * @see https://recoiljs.org/docs/api-reference/core/useRecoilState/
 */
function useAtom(atom) {
  var state = (0, _react.useSyncExternalStore)(atom.subscribe, atom.getState, atom.getState);
  return [state, atom.setState];
}